library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity upl_output_port is
  port (
    UPLGlobalClk : in std_logic;
    UPLReset : in std_logic;
    ready : out std_logic;
    kick : in std_logic;
    send_length : in signed(32-1 downto 0);
    data_length : out signed(32-1 downto 0);
    data_address : in signed(32-1 downto 0);
    data_dout : out signed(32-1 downto 0);
    data_din : in signed(32-1 downto 0);
    data_we : in std_logic;
    data_oe : in std_logic;
    UPLOut_Data : out std_logic_vector(32-1 downto 0);
    UPLOut_En : out std_logic;
    UPLOut_Request : out std_logic;
    UPLOut_Ack : in std_logic
  );
end upl_output_port;

architecture RTL of upl_output_port is

  attribute mark_debug : string;
  attribute keep : string;
  attribute S : string;

  component dualportram
    generic (
      WIDTH : integer := 32;
      DEPTH : integer := 10;
      WORDS : integer := 1024
    );
    port (
      clk : in std_logic;
      reset : in std_logic;
      length : out signed(31 downto 0);
      address : in signed(31 downto 0);
      din : in signed(WIDTH-1 downto 0);
      dout : out signed(WIDTH-1 downto 0);
      we : in std_logic;
      oe : in std_logic;
      address_b : in signed(31 downto 0);
      din_b : in signed(WIDTH-1 downto 0);
      dout_b : out signed(WIDTH-1 downto 0);
      we_b : in std_logic;
      oe_b : in std_logic
    );
  end component dualportram;

  signal UPLGlobalClk_sig : std_logic := '0';
  signal UPLReset_sig : std_logic := '0';
  signal ready_sig : std_logic := '0';
  signal kick_sig : std_logic := '0';
  signal send_length_sig : signed(32-1 downto 0) := (others => '0');
  signal data_length_sig : signed(32-1 downto 0) := (others => '0');
  signal data_address_sig : signed(32-1 downto 0) := (others => '0');
  signal data_dout_sig : signed(32-1 downto 0) := (others => '0');
  signal data_din_sig : signed(32-1 downto 0) := (others => '0');
  signal data_we_sig : std_logic := '0';
  signal data_oe_sig : std_logic := '0';
  signal UPLOut_Data_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal UPLOut_En_sig : std_logic := '0';
  signal UPLOut_Request_sig : std_logic := '0';
  signal UPLOut_Ack_sig : std_logic := '0';

  signal U_RAM_clk : std_logic := '0';
  signal U_RAM_reset : std_logic := '0';
  signal U_RAM_length : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_address : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_din : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_dout : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_we : std_logic := '0';
  signal U_RAM_oe : std_logic := '0';
  signal U_RAM_address_b : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_din_b : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_dout_b : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_we_b : std_logic := '0';
  signal U_RAM_oe_b : std_logic := '0';
  signal local_rdata : signed(32-1 downto 0) := (others => '0');
  signal local_addr : signed(32-1 downto 0) := (others => '0');
  signal local_din : signed(32-1 downto 0) := (others => '0');
  signal local_we : std_logic := '0';
  signal local_oe : std_logic := '0';
  signal send_count : signed(32-1 downto 0) := (others => '0');
  type Type_main is (
    main_IDLE,
    main_OPERATION,
    main_SEND_WAIT,
    main_MEM_WAIT,
    main_SEND_DATA  
  );
  signal main : Type_main := main_IDLE;
  signal main_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0002 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0003 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0004 : signed(32-1 downto 0) := (others => '0');
  signal kick_d : std_logic := '0';
  signal tmp_0005 : std_logic := '0';
  signal tmp_0006 : std_logic := '0';
  signal tmp_0007 : std_logic := '0';
  signal tmp_0008 : std_logic := '0';
  signal tmp_0009 : std_logic := '0';
  signal tmp_0010 : std_logic := '0';
  signal tmp_0011 : std_logic := '0';

begin

  UPLGlobalClk_sig <= UPLGlobalClk;
  UPLReset_sig <= UPLReset;
  ready <= ready_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        ready_sig <= '0';
      else
        if main = main_OPERATION then
          ready_sig <= '1';
        else
          ready_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  kick_sig <= kick;
  send_length_sig <= send_length;
  data_length <= data_length_sig;
  data_length_sig <= U_RAM_length;

  data_address_sig <= data_address;
  data_dout <= data_dout_sig;
  data_dout_sig <= U_RAM_dout;

  data_din_sig <= data_din;
  data_we_sig <= data_we;
  data_oe_sig <= data_oe;
  UPLOut_Data <= UPLOut_Data_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLOut_Data_sig <= (others => '0');
      else
        if main = main_SEND_DATA then
          UPLOut_Data_sig <= std_logic_vector(local_rdata);
        end if;
      end if;
    end if;
  end process;

  UPLOut_En <= UPLOut_En_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLOut_En_sig <= '0';
      else
        if main = main_SEND_DATA then
          UPLOut_En_sig <= '1';
        else
          UPLOut_En_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  UPLOut_Request <= UPLOut_Request_sig;
  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        UPLOut_Request_sig <= '0';
      else
        if main = main_SEND_WAIT then
          UPLOut_Request_sig <= '1';
        else
          UPLOut_Request_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  UPLOut_Ack_sig <= UPLOut_Ack;

  -- expressions
  tmp_0001 <= send_length_sig;
  tmp_0002 <= local_addr + X"00000001";
  tmp_0003 <= send_count - X"00000001";
  tmp_0004 <= local_addr + X"00000001";
  tmp_0005 <= '1' when kick_sig = '1' else '0';
  tmp_0006 <= '1' when kick_d = '0' else '0';
  tmp_0007 <= tmp_0005 and tmp_0006;
  tmp_0008 <= '1' when send_length_sig > X"00000000" else '0';
  tmp_0009 <= tmp_0007 and tmp_0008;
  tmp_0010 <= '1' when UPLOut_Ack_sig = '1' else '0';
  tmp_0011 <= '1' when send_count = X"00000001" else '0';

  -- sequencers
  process (UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        main <= main_IDLE;
        main_delay <= (others => '0');
      else
        case (main) is
          when main_IDLE => 
            main <= main_OPERATION;
          when main_OPERATION => 
            if tmp_0009 = '1' then
              main <= main_SEND_WAIT;
            end if;
          when main_SEND_WAIT => 
            if tmp_0010 = '1' then
              main <= main_MEM_WAIT;
            end if;
          when main_MEM_WAIT => 
            main <= main_SEND_DATA;
          when main_SEND_DATA => 
            if tmp_0011 = '1' then
              main <= main_OPERATION;
            end if;
          when others => null;
        end case;
        kick_d <= kick_sig;
      end if;
    end if;
  end process;


  U_RAM_clk <= UPLGlobalClk_sig;

  U_RAM_reset <= UPLReset_sig;

  U_RAM_address <= data_address_sig;

  U_RAM_din <= data_din_sig;

  U_RAM_we <= data_we_sig;

  U_RAM_oe <= data_oe_sig;

  U_RAM_address_b <= local_addr;

  U_RAM_din_b <= local_din;

  U_RAM_we_b <= local_we;

  U_RAM_oe_b <= local_oe;

  local_rdata <= U_RAM_dout_b;

  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        local_addr <= (others => '0');
      else
        if main = main_SEND_WAIT then
          local_addr <= X"00000000";
        elsif main = main_MEM_WAIT then
          local_addr <= tmp_0002;
        elsif main = main_SEND_DATA then
          local_addr <= tmp_0004;
        end if;
      end if;
    end if;
  end process;

  process(UPLGlobalClk)
  begin
    if UPLGlobalClk'event and UPLGlobalClk = '1' then
      if UPLReset = '1' then
        send_count <= (others => '0');
      else
        if main = main_SEND_WAIT then
          send_count <= tmp_0001;
        elsif main = main_SEND_DATA then
          send_count <= tmp_0003;
        end if;
      end if;
    end if;
  end process;


  inst_U_RAM : dualportram
  generic map(
    WIDTH => 32,
    DEPTH => 10,
    WORDS => 1024
  )
  port map(
    clk => UPLGlobalClk,
    reset => UPLReset,
    length => U_RAM_length,
    address => U_RAM_address,
    din => U_RAM_din,
    dout => U_RAM_dout,
    we => U_RAM_we,
    oe => U_RAM_oe,
    address_b => U_RAM_address_b,
    din_b => U_RAM_din_b,
    dout_b => U_RAM_dout_b,
    we_b => U_RAM_we_b,
    oe_b => U_RAM_oe_b
  );


end RTL;
